package de.fhdw.gaming.ipspiel23.memory;

/**
 * Interface for a game memory that stores and retrieves round data.
 * 
 * @param <ROUND_DATA> the type of round data to be stored.
 */
public interface IGameMemory<ROUND_DATA> {

    /**
     * number of rounds that are currently stored, must be >= 0 (duh).
     */
    int size();

    /**
     * maximum possible number of rounds that can be stored.
     * -1 means unlimited.
     */
    int capacity();

    /**
     * adds a round, potentially overriding the oldest round data, if capacity() is reached.
     * 
     * @param round the round data to add.
     */
    void add(ROUND_DATA round);

    /**
     * resets this memory instance to its initial state.
     */
    void clear();

    /**
     * returns the round at the specified index relative to the oldest round in the data set,
     * such that index = 0, fromEnd = false will always return the oldest round (furthest in the past), 
     * while index = 0, fromEnd = true will return the latest round (the round that just happened).
     * The index must always be 0 <= index < capacity()
     * 
     * @param index the index of the round data to retrieve.
     * @param fromEnd if true, retrieves the round data from the end of the buffer.
     * @return the round data at the specified index, or null if the buffer is empty.
     */
    ROUND_DATA getRound(int index, boolean fromEnd);

    /**
     * Returns the round data at the specified index.
     * Counted from the end of the buffer.
     * 
     * @param index the index of the round data to retrieve.
     * @return the round data at the specified index, or null if the buffer is
     *         empty.
     */
    ROUND_DATA getRound(int index);

    /**
     * Returns the latest round data in the buffer.
     * 
     * @return the latest round data in the buffer, or null if the buffer is empty.
     */
    ROUND_DATA getLatestRound();
}
